module SSG.Djot.Render

import SSG.HTML

import SSG.Djot.Inline
import SSG.Djot.Block

import Data.String
import Data.List1
import Data.List

import Control.Monad.Eval
import Structures.Dependent.DList

-- Maybe because specifically Soft line breaks don't generate any html of their
-- own
export
renderInline : Inline -> Maybe (type : String ** Html type)
renderInline HardLineBreak =
  Just (_ ** Void "br" [])
renderInline SoftLineBreak =
  Nothing
renderInline NonBreakingSpace =
  Just (_ ** Text "&nbsp;")
renderInline (Text c) =
  Just (_ ** Text c)

combineTexts : (types : List String ** DList _ Html types)
  -> Eval (types : List String ** DList _ Html types)
combineTexts (_ ** []) = pure (_ ** [])
-- We do a little bit of magic insert of whitespace, so we have some special handling for
-- nbsps to not insert spaces around them
combineTexts (_ ** (Text c :: Text "&nbsp;" :: Text "&nbsp;" :: rest)) =
  combineTexts (_ ** Text (c ++ "&nbsp;") :: Text "&nbsp;" :: rest)
combineTexts (_ ** (Text c :: Text "&nbsp;" :: Text d :: rest)) =
  combineTexts (_ ** Text (c ++ "&nbsp;" ++ d) :: rest)
combineTexts (_ ** (Text c :: Text "&nbsp;" :: rest)) =
  combineTexts (_ ** Text (c ++ "&nbsp;") :: rest)
combineTexts (_ ** (Text c :: Text d :: rest)) =
  combineTexts (_ ** Text (c ++ " " ++ d) :: rest)
combineTexts (_ ** (elem :: rest)) = do
  (_ ** rest) <- combineTexts (_ ** rest)
  pure $ (_ ** elem :: rest)

export
renderInlines : List Inline -> (types : List String ** DList _ Html types)
renderInlines xs =
  eval . combineTexts . fromList . catMaybes . map renderInline $ xs

headingLevel : HeaderLevel -> (h : String ** IsNormal h)
headingLevel H1 = ("h1" ** IsH1)
headingLevel H2 = ("h2" ** IsH2)
headingLevel H3 = ("h3" ** IsH3)
headingLevel H4 = ("h4" ** IsH4)
headingLevel H5 = ("h5" ** IsH5)
headingLevel H6 = ("h6" ** IsH6)

export
renderBlock : Block -> (type : String ** Html type)
renderBlock (Paragraph contents) =
  let (_ ** xs) = renderInlines $ forget contents
  in (_ ** Normal "p" [] xs)
renderBlock (Heading level contents) =
  let (_ ** xs) = renderInlines $ forget contents
      (level ** _ ) = headingLevel level
  in (_ ** Normal level [] xs)

export
renderBlocks : List Block -> (types : List String ** DList _ Html types)
renderBlocks xs = fromList $ map renderBlock xs

export
renderHtml : List Block -> Html "html"
renderHtml xs =
  let (_ ** xs) = renderBlocks xs
  in Normal "html" ["lang" =$ "en"]
    [ Normal "body" [] xs ]
